#include "gibbsMedLDA.h"
#include "cokus.h"
#include "svmlight/svm_common.h"
#include "utils.h"
#include <math.h>
#include <assert.h>
#include <stdio.h>
#include <string.h>
#include <vector>
#include <map>
#include <time.h>

using namespace std;

void TopK::reset()
{
	memset(key, 0, sizeof(int) * length);
	memset(value, 0, sizeof(double) * length);
}

void TopK::push_back(int id, double v)
{
	if (v > value[0]) 
	{
		value[0] = v;
		key[0] = id;
		shiftdown(0, length);
	}
}

void TopK::shiftdown(int pos, int rg)
{
	int left = 2 * pos + 1;
	int right = left + 1;
	int min = pos;
	if (left < rg && value[left] < value[pos])
		min = left;		
	if (right < rg && value[right] < value[min])
		min = right;
	
	if (min != pos)
	{
		exchange(min, pos);
		shiftdown(min, rg);
	}
}

void TopK::sort()
{
	int l = length;
	while(l--)
	{
		exchange(0, l);
		shiftdown(0, l);
	}
}

inline void TopK::exchange(int p1, int p2)
{
	int k = key[p1];
	double v = value[p1];
	key[p1] = key[p2];
	value[p1] = value[p2];
	key[p2] = k;
	value[p2] = v;
}

//////////////////////////////////////////////////////////

Stats::Stats()
{
	nD			= 0;
	nV			= 0;
	nK			= 0;
	z			= NULL;
	nw			= NULL;
	nwsum		= NULL;
	nd			= NULL;
	corpus 	= NULL;
}

Stats::~Stats()
{
	Stats::free();
}

void Stats::free()
{
	if (z)
	{
		for (int i = 0; i < nD; ++i)
			if (z[i]) delete[] z[i];
		delete[] z;
	}
	z = NULL;

	if (nw)
	{
		for (int i = 0; i < nK; ++i)
			if (nw[i]) delete[] nw[i];
		delete[] nw;
	}
	nw = NULL;

	if (nwsum) delete[] nwsum;
	nwsum = NULL;

	if (nd)
	{
		for(int i = 0; i < nD; ++i)
			if (nd[i]) delete[] nd[i];
		delete[] nd;
	}
	nd = NULL;

	if (corpus) delete corpus;
	corpus = NULL;

	nD = 0;
	nV	= 0;
	nK	= 0;
}

void Stats::new_stats(Corpus* cps, Params* param)
{
	Stats::free();

	corpus 	= cps;
	nD			= corpus->num_docs;
	nV			= corpus->num_terms;
	nK			= param->NTOPICS;

	z = new int*[nD];
	for(int d = 0; d < nD; ++d)
	{
		Document& dc = corpus->docs[d];
		z[d] = new int[dc.length];
	}

	nwsum = new int[nK];
	memset(nwsum, 0, sizeof(int) * nK);

	nw = new int*[nK];
	for(int k = 0; k < nK; ++k)
	{
		nw[k] = new int[nV];
		memset(nw[k], 0, sizeof(int) * nV);
	}

	nd = new int*[nD];
	for(int d = 0; d < nD; ++d)
	{
		nd[d] = new int[nK];	
		memset(nd[d], 0, sizeof(int) * nK);
	}
}

void Stats::load(const char* filename, Params* param)
{
	FILE* fin = fopen(filename, "r");
	load(fin, param);
	fclose(fin);
}

void Stats::load(FILE* file, Params* param)
{
	Stats::free();

	int length, label, word, topic;
	int nword = 0;

	nK	= param->NTOPICS;

	corpus = new Corpus();
	fscanf(file, "%d\n", &nD);
	corpus->num_docs = nD;
	corpus->docs = new Document[nD];

	z = new int*[nD];
	nd = new int*[nD];
	for(int d = 0; d < nD; ++d)
	{
		nd[d] = new int[nK];	
		memset(nd[d], 0, sizeof(int) * nK);

		Document& dc = corpus->docs[d];
		fscanf(file, "%d %d", &length, &label);
		dc.length	= length;
		dc.gndlabel	= label;
		dc.words		= new int[length];
		z[d]	 		= new int[length];
		for(int n = 0; n < length; ++n)
		{
			fscanf(file, "%d:%d", &word, &topic);
			dc.words[n]		= word;
			z[d][n]			= topic;
			nd[d][topic] 	+= 1;
			if (word >= nword ) nword = word + 1;
		}
	}
	corpus->num_terms = nV = nword;

	nw = new int*[nK];
	for(int k = 0; k < nK; ++k)
	{
		nw[k] = new int[nV];
		memset(nw[k], 0, sizeof(int) * nV);
	}
	nwsum = new int[nK];
	memset(nwsum, 0, sizeof(int) * nK);

	for(int d = 0; d < nD; ++d)
	{
		Document& dc = corpus->docs[d];
		length = dc.length; 
		for(int n = 0; n < length; ++n)
		{
			topic = z[d][n]; 
			nw[topic][dc.words[n]] += 1;
			nwsum[topic] += 1;
		}
	}
}

void Stats::save_wordassigned(const char *filename)
{
	printf("Word-Topic Assignments: %s\n",filename);

	FILE* fout = fopen(filename, "w");
	save_wordassigned(fout);
	fclose(fout);
}

void Stats::save_wordassigned(FILE *file)
{
	if (!file) return;
	fprintf(file, "%d\n", nD);
	for (int d = 0; d < nD; ++d)
	{
		Document& dc = corpus->docs[d];
		fprintf(file, "%d %d", dc.length, dc.gndlabel);	
		for(int n = 0; n < dc.length; ++n)
			fprintf(file, " %d:%d", dc.words[n], z[d][n]);
		fprintf(file, "\n");
	}
}

void Stats::save_topwords(const char* filename, Params* param) 
{
	FILE* fin = fopen(param->mapper_filename, "r");
	if (!fin) return;

	char buf[512];
	map<int, string> mapper;
	int id;
	while( fscanf(fin, "%d %s\n", &id, buf) != EOF )
		mapper.insert(map<int, string>::value_type(id, string(buf)));
	fclose(fin);

	map<int, string>::iterator it;
	FILE* fout = fopen(filename, "w");
	if (!fout) return;

	TopK topk(param->SAVE_TOP_WORDS);
	for(int k = 0; k < nK; ++k)
	{
		topk.reset();
		for(int w = 0; w < nV; ++w)
			topk.push_back(w, (double) nw[k][w] / (double) nwsum[k]);
		topk.sort();
		fprintf(fout, "Topic %d:", k);
		for(int i = 0; i < topk.length; ++i)
		{
			it = mapper.find(topk.key[i]);
			if (mapper.end() != it)
				fprintf(fout, " %s:%f", it->second.c_str(), topk.value[i]);
		}
		fprintf(fout, "\n");
	}
	fclose(fout);
}

void Stats::save_label_only(const char* filename)
{
	FILE* fout = fopen(filename, "w");
	fprintf(fout, "%d\n", nD);
	for(int d = 0; d < nD; ++d)
	{
		Document& dc = corpus->docs[d];
		fprintf(fout, "%d %d\n", dc.gndlabel, dc.predlabel); 
	}
	fclose(fout);
}

//////////////////////////////////////////////////////////

void GibbsMedLDA::set_default_values()
{
	m_ptrnstats			= NULL;
	m_ptststats			= NULL;

	m_nK					= 0; // number of topics, K
	m_nV					= 0;
	m_nLabelNum			= 0; // number of classes, |C|
	m_dAlpha				= 0.0; 
	m_dBeta				= 0.0;
	m_dMu					= NULL;
	m_dEta				= NULL; 
	m_dC					= 0.0;
	m_dB					= 0.0;
	m_dsvm_primalobj	= 0.0;

	m_dPr					= NULL; 
	m_mrg					= NULL;
	m_dPhi				= NULL;
	m_dTheta				= NULL;
}

GibbsMedLDA::~GibbsMedLDA()
{
	if (m_dMu) delete[] m_dMu;
	m_dMu = NULL;

	if (m_dEta) delete[] m_dEta;
	m_dEta = NULL;

	if (m_dPr) delete[] m_dPr;
	m_dPr = NULL;

	if (m_mrg) delete[] m_mrg;
	m_mrg = NULL;

	if (m_dPhi)
	{
		for(int i = 0; i < m_nK; ++i)
			delete[] m_dPhi[i];
		delete[] m_dPhi;
	}
	m_dPhi = NULL;

	if (m_dTheta) delete[] m_dTheta;
	m_dTheta = NULL;

	if (m_ptrnstats) delete m_ptrnstats;
	m_ptrnstats = NULL;

	if (m_ptststats) delete m_ptststats;
	m_ptststats	= NULL;
}

void GibbsMedLDA::init_est()
{
	Corpus* corpus = new Corpus();
	corpus->read_data(m_params->train_filename);
	init_est(corpus);
}

void GibbsMedLDA::init_est(Corpus* corpus)
{
	m_nK 			= m_params->NTOPICS;
	m_nLabelNum = m_params->NLABELS,
	m_dC			= m_params->INITIAL_C;
	m_dAlpha 	= m_params->INITIAL_ALPHA / m_params->NTOPICS;
	m_dBeta 		= m_params->INITIAL_BETA; 

	m_ptrnstats = new Stats();
	m_ptrnstats->new_stats(corpus, m_params);
	m_nV = m_ptrnstats->nV;

	const int m_nD = m_ptrnstats->nD;
	int **m_nw		= m_ptrnstats->nw;
	int *m_nwsum	= m_ptrnstats->nwsum;
	int **m_nd		= m_ptrnstats->nd;

	if (!m_dMu) m_dMu = new double[m_nD * m_nLabelNum];
	memset(m_dMu, 0, sizeof(double) * m_nD * m_nLabelNum);

	if (!m_dEta) m_dEta = new double[m_nLabelNum * m_nK];
	memset(m_dEta, 0, sizeof(double) * m_nLabelNum * m_nK);

	if (!m_dPr) m_dPr = new double[m_nK];
	if (!m_mrg) m_mrg = new double[m_nK];

	int N, topic;
	int* zptr = NULL;
	for(int d = 0; d < m_nD; ++d)
	{
		Document& dc = corpus->docs[d];
		N = dc.length; 
		zptr = m_ptrnstats->z[d];
		for(int n = 0; n < N; ++n)
		{
			topic = (int)(myrand() * m_nK);	
			*zptr++ = topic;
			m_nw[topic][dc.words[n]] += 1;
			m_nwsum[topic] += 1;
			m_nd[d][topic] += 1;
		}
	}
}

void GibbsMedLDA::init_estc(const char* prev_state_file)
{
	m_nK 			= m_params->NTOPICS; 
	m_nLabelNum = m_params->NLABELS,
	m_dC			= m_params->INITIAL_C;
	m_dAlpha 	= m_params->INITIAL_ALPHA / m_params->NTOPICS;
	m_dBeta 		= m_params->INITIAL_BETA; 

	if (!m_ptrnstats) load_model(prev_state_file);

	if (!m_dPr) m_dPr = new double[m_nK];
	if (!m_mrg) m_mrg = new double[m_nK];
}

void GibbsMedLDA::init_inf(Corpus* corpus)
{
	m_nK			= m_params->NTOPICS;
	m_nLabelNum = m_params->NLABELS,
	m_dC			= m_params->INITIAL_C;
	m_dAlpha 	= m_params->INITIAL_ALPHA / m_params->NTOPICS;
	m_dBeta 		= m_params->INITIAL_BETA; 

	if (m_ptststats) delete m_ptststats;
	m_ptststats = new Stats();
	m_ptststats->new_stats(corpus, m_params);
	if (m_ptststats->nV > m_nV) 
		m_nV = m_ptststats->nV;

	if(!m_dPr) m_dPr = new double[m_nK];

	const int infD	= m_ptststats->nD;
	int **infnw		= m_ptststats->nw;
	int *infnwsum	= m_ptststats->nwsum;
	int *zptr 		= NULL;
	int *infnd 		= NULL;
	int topic, term;
	for (int d = 0; d < infD; ++d)
	{
		Document& dc 	= corpus->docs[d];
		zptr				= m_ptststats->z[d];
		infnd				= m_ptststats->nd[d];
		for( int n = 0; n < dc.length; ++n)
		{
			term = dc.words[n];
			topic = (int)(myrand() * m_nK);
			*zptr++ = topic;
			infnw[topic][term] 	+= 1;
			infnwsum[topic]		+= 1;
			infnd[topic]			+= 1;
		}
	}
}

void GibbsMedLDA::init_inf(const char* model_file)
{
	load_model(model_file);

	Corpus* corpus = new Corpus();
#ifdef _REMOVE_OOV_
	corpus->read_data(m_params->test_filename, m_nV);
#else
	corpus->read_data(m_params->test_filename);
#endif
	init_inf(corpus);
}

void GibbsMedLDA::run_em()
{
	char buf[512];
	double lhood = 1;
	double lhood_old = 0;
	double converged = 1;
	int iter = 0;

	double svmTime = 0.0;
	time_t rawtime;
	struct tm *timeinfo = NULL;
	time(&rawtime);
	timeinfo = localtime(&rawtime);
	printf("K: %d >> Training start time: %s\n", m_nK, asctime(timeinfo));
	fflush(stdout);

	if (m_params->EM_CONVERGED >= 0)
	{
		sprintf(buf, "%s/train_likelihood.txt", m_params->dir);
		FILE* flhood = fopen(buf, "w");
		sprintf(buf, "%s/ulda.model", m_params->dir);
		while((converged > m_params->EM_CONVERGED) && (iter < m_params->EM_MAX_ITER) || (m_params->EM_MAX_ITER < 0))
		{
			printf("----- %d-th EM ------\n", iter);
			lhood = e_step(iter);
			fflush(stdout);

			clock_t start = clock();
			m_step();
			clock_t end = clock();
			svmTime += double(end - start)/CLOCKS_PER_SEC;
			fflush(stdout);
			//if (0 == iter) save_model(buf);
			printf("----------------------------\n");
			printf("|  likelihood = %.10f", lhood);
			fprintf(flhood, "likelihood:%.10f", lhood);
			lhood += m_dsvm_primalobj;
			printf("\t objective_funtion= %.10f\n", lhood);
			fprintf(flhood, "\t objective_function:%.10f\n", lhood);
			printf("----------------------------\n");
			converged = lhood_old - lhood;
			converged = fabs(converged/lhood_old);
			lhood_old = lhood;
			printf("--- %d-th EM finished ----\n", iter);
			iter++;
		}
		fclose(flhood);
	}
	else
	{
		for(iter = 0; iter < m_params->EM_MAX_ITER; ++iter)
		{
			printf("----- %d-th EM ------\n", iter);
			e_step(iter);
			m_step();
			printf("--- %d-th EM finished ----\n", iter);
		}
	}

	sprintf(buf, "%s/time.txt", m_params->dir);
	FILE *ftime = fopen(buf, "w");
	fprintf(ftime, "K: %d\n", m_nK);
	fprintf(ftime, "Training start time: %s\n", asctime(timeinfo)); 
	time(&rawtime);
	timeinfo = localtime(&rawtime);
	fprintf(ftime, "Training end time: %s\n", asctime(timeinfo)); 
	fprintf(ftime, "Training SVM time: %lf\n", svmTime); 
	fclose(ftime);
	printf("K: %d >> Training end time: %s\n", m_nK, asctime(timeinfo)); 
	printf("K: %d >> Training SVM time: %lf\n", m_nK, svmTime); 
	fflush(stdout);

	printf("Final E-step\n");
	lhood = e_step(iter);
	printf("Final E-step Likelihood: %5.10f\n", lhood);
	printf("Final E-step finished!\n");

	sprintf(buf, "%s/topwords.txt", m_params->dir);
	m_ptrnstats->save_topwords(buf, m_params);

	int nAcc = 0;
	int label = 0;
	for (int d = 0; d < m_ptrnstats->nD; ++d)
	{
		label = predict(m_ptrnstats, d);	
		Document& dc = m_ptrnstats->corpus->docs[d];
		dc.predlabel = label;
		if (dc.gndlabel == label) ++nAcc;
	}
	printf("\n\n Double count accuracy: %.5f\n\n", (double)nAcc/m_ptrnstats->nD);
	sprintf(buf, "%s/double_count_accuracy.txt", m_params->dir);
	FILE* dfile = fopen(buf, "w");
	fprintf(dfile, "Double count accuracy: %.5f\n", (double)nAcc/m_ptrnstats->nD);
	fclose(dfile);


}

double GibbsMedLDA::e_step(int em_it)
{
	char model_file[500];
	const int m_nD = m_ptrnstats->nD;
	double lhood = 0;
	double lhood_old = 0;
	double converged = 1;
	int *zptr = NULL;
	bool log_space_flag = false;

	printf("gibbs_conv: %.10lf\tmax_iters:%d\n", m_params->GIBBS_CONVERGED, m_params->NITERS);
	if (m_params->GIBBS_CONVERGED >= 0)
	{
		int iter = 0;
		while((iter < MIN_GIBBS_ITERATION) || (converged > m_params->GIBBS_CONVERGED) && (iter < m_params->NITERS) || (m_params->NITERS < 0))
		{
			printf("Iteration %d ..., ", iter);
			for(int d = 0; d < m_nD; ++d)
			{
				Document& dc = m_ptrnstats->corpus->docs[d];
				zptr = m_ptrnstats->z[d]; 
				log_space_flag = compute_mrgterm(d, dc.gndlabel, dc.length, m_mrg);
				if (log_space_flag) {
					for(int n = 0; n < dc.length; ++n)
						*zptr++ = gibbs_sampling_logspace(m_ptrnstats, m_mrg, d, n);
				} else {
					for(int n = 0; n < dc.length; ++n)
						*zptr++	= gibbs_sampling(m_ptrnstats, m_mrg, d, n);
				}
			}
			if ((m_params->ITER_SAVE_STEP > 0) && (iter % m_params->ITER_SAVE_STEP == 0)) {
				sprintf(model_file, "%s/model-%d-%d", m_params->dir, em_it, iter);
				save_model(model_file);
			}
			lhood = compute_lhood(m_ptrnstats);
			converged = (lhood_old - lhood);
			converged = fabs(converged/lhood_old);
			printf("\tlhood: %f\tdelta: %f\n", lhood, converged);
			lhood_old = lhood;
			++iter;
		}
		printf("Done! \tlhood: %f\tdelta: %f\n", lhood, converged);
	}
	else;

	return lhood;
}

double GibbsMedLDA::inference(double *tAvgEnt)
{
	char buf[512];
	sprintf(buf, "%s/inf_single_likelihood.txt", m_params->dir);

	int nAcc	= 0;

	double converged  = 1;
	double lhood = 0, lhood_old = 0;
	double lhood_all = 0;
	double avgEnt = 0, dEnt, val;
	int *zptr 			= NULL;
	int zLen, n, iter, topic, k;

	const int infD 	= m_ptststats->nD;
	const int infV		= m_ptststats->nV;
	Document* docs 	= m_ptststats->corpus->docs;
	int** infnw			= m_ptststats->nw;
	int* infnwsum		= m_ptststats->nwsum;

	if (m_dPhi) {
		for(k = 0; k < m_nK; ++k) {
			if (m_dPhi[k]) delete[] m_dPhi[k];
			m_dPhi[k] = NULL;
		}
		delete[] m_dPhi;
	}
	m_dPhi = NULL;

	if (m_params->GIBBS_CONVERGED >= 0) 
		compute_phi(m_ptrnstats);
	int *suff_topics = new int[m_nK];

	time_t rawtime;
	struct tm *timeinfo;
	time(&rawtime);
	timeinfo = localtime(&rawtime);
	printf("K: %d >> Test start time: %s\n", m_nK, asctime(timeinfo));
	fflush(stdout);

	printf("Prediction Inference ...\n");
	for(int d = 0; d < infD; ++d) {
		memset(infnwsum, 0, sizeof(int) * m_nK);
		for(k = 0; k < m_nK; ++k)
			memset(infnw[k], 0, sizeof(int)* infV);

		Document& dc = docs[d];
		zLen = dc.length; 
		zptr = m_ptststats->z[d];
		for(n = 0; n < zLen; ++n)
		{
			topic = zptr[n];
			infnw[topic][dc.words[n]] += 1;
			infnwsum[topic] += 1;
		}


		if (d % 1000 == 0) printf("Document %d\n", d);

		iter = 0;
		lhood = 0;
		if (m_params->GIBBS_CONVERGED >= 0) {
			converged  = 1;
			lhood_old = 0;
		  	while((converged > m_params->GIBBS_CONVERGED) && (iter < m_params->NITERS) || (m_params->NITERS < 0)) {
		  		for(n = 0; n < zLen; ++n)
		  			zptr[n] = gibbs_sampling(m_ptrnstats, m_ptststats, d, n);
				compute_theta(m_ptststats, d);
		  		lhood = compute_lhood(&dc, m_dPhi, m_dTheta);
		  		converged = (lhood_old - lhood);
		  		converged = fabs(converged/lhood_old);
		  		lhood_old = lhood;
		  		++iter;
		  	}
		} else {
			for (iter = 0; iter < m_params->NITERS; ++iter)
			  	for(n = 0; n < zLen; ++n)
			  		zptr[n] = gibbs_sampling(m_ptrnstats, m_ptststats, d, n);
		}
		lhood_all += lhood;
		dc.lhood  = lhood;

		//Predicting labels based on the sufficient statistics
		memset(suff_topics, 0, sizeof(int) * m_nK);
		for (iter = 0; iter < m_params->SAMPLING_LAG; ++iter) {
		  	for(n = 0; n < zLen; ++n) {
				topic = gibbs_sampling(m_ptrnstats, m_ptststats, d, n);
				zptr[n] = topic;
				suff_topics[topic] += 1;
			}
		}

		if (m_params->SAMPLING_LAG > 1){
			dEnt = 0;
			for(k = 0; k < m_nK; ++k) {
				val = (double)(suff_topics[k]/(double)m_params->SAMPLING_LAG + m_dAlpha)/(dc.length + m_nK * m_dAlpha);
				if (val > 0) dEnt -= val * log(val);
				m_dTheta[k] = val;
			}
			avgEnt += dEnt;
		} else if (m_params->SAMPLING_LAG == 1){
			double DV = (double)dc.length + m_nK * m_dAlpha;
			dEnt = 0;
			for(k = 0; k < m_nK; ++k) {
				val = ((double)suff_topics[k]+ m_dAlpha)/DV;
				dEnt -= val * log(val);
				m_dTheta[k] = val;
			}
			avgEnt += dEnt;
		} else ;

		dc.predlabel = predict(m_dTheta);
		if (dc.predlabel == dc.gndlabel ) ++nAcc;
	}


	sprintf(buf, "%s/time.txt", m_params->dir);
	FILE *ftime = fopen(buf, "a");
	fprintf(ftime, "Test start time: %s\n", asctime(timeinfo));
	time(&rawtime);
	timeinfo = localtime(&rawtime);
	fprintf(ftime, "Test end time: %s\n", asctime(timeinfo));
	fclose(ftime);
	printf("K: %d >> Test end time: %s\n", m_nK, asctime(timeinfo));
	fflush(stdout);

	printf("Likilihood_ALL : %5.10f\n", lhood_all);
	printf("Prediction Inference FINISHED !\n");
	*tAvgEnt = avgEnt / infD;
	delete[] suff_topics;
	return (double) nAcc / infD;
}

int GibbsMedLDA::gibbs_sampling_logspace(Stats* ss, const double* mrg, const int d, const int n)
{
	Document& dc		= ss->corpus->docs[d];
	const int term		= dc.words[n];
	const int y			= dc.gndlabel;
	const int len		= dc.length;
	int **m_z			= ss->z;
	int **m_nw			= ss->nw;
	int *m_nwsum		= ss->nwsum;
	int **m_nd			= ss->nd;

	int topic 			= m_z[d][n];
	double Vbeta		= m_nV * m_dBeta;

	m_nw[topic][term] -= 1;
	m_nwsum[topic]		-= 1;
	m_nd[d][topic]		-= 1;

	for(int k = 0; k < m_nK; ++k)
		m_dPr[k] = log((m_nw[k][term] + m_dBeta) / (m_nwsum[k] + Vbeta) * (m_nd[d][k] + m_dAlpha)) + mrg[k];
	double logsum = log_sum(m_dPr, m_nK);

	m_dPr[0] = exp(m_dPr[0] - logsum);
	for(int k = 1; k < m_nK; ++k)
		m_dPr[k] = m_dPr[k-1] + exp(m_dPr[k] - logsum); 

	double rd = myrand() * m_dPr[m_nK-1];
	for(topic = 0; m_dPr[topic] < rd; ++topic);

	m_nw[topic][term]	+= 1;
	m_nwsum[topic]		+= 1;
	m_nd[d][topic]		+= 1;

	return topic;
}

int GibbsMedLDA::gibbs_sampling(Stats* ss, const double* mrg, const int d, const int n)
{
	Document& dc		= ss->corpus->docs[d];
	const int term		= dc.words[n];
	const int y			= dc.gndlabel;
	const int len		= dc.length;
	int **m_z			= ss->z;
	int **m_nw			= ss->nw;
	int *m_nwsum		= ss->nwsum;
	int **m_nd			= ss->nd;

	int topic 			= m_z[d][n];
	double Vbeta		= m_nV * m_dBeta;

	m_nw[topic][term] -= 1;
	m_nwsum[topic]		-= 1;
	m_nd[d][topic]		-= 1;

	m_dPr[0] = (m_nw[0][term] + m_dBeta) / (m_nwsum[0] + Vbeta) * (m_nd[d][0] + m_dAlpha) * exp(mrg[0]);
	for(int k = 1; k < m_nK; ++k)
		m_dPr[k] = m_dPr[k-1] + (m_nw[k][term] + m_dBeta) / (m_nwsum[k] + Vbeta) * (m_nd[d][k] + m_dAlpha) * exp(mrg[k]); 

	double rd = myrand() * m_dPr[m_nK-1];
	for(topic = 0; m_dPr[topic] < rd; ++topic);

	m_nw[topic][term]	+= 1;
	m_nwsum[topic]		+= 1;
	m_nd[d][topic]		+= 1;

	return topic;
}

inline bool GibbsMedLDA::compute_mrgterm(const int d, const int y, const int dlen, double* mrg)
{
	int t;
	double dval, gnd_eta, ov;
	double *mu_ptr = NULL; 
	double *eta_ptr = NULL; 
	bool flag = false;
	for(int k = 0; k < m_nK; ++k)
	{
		dval = 0;
		t = m_nLabelNum;
		mu_ptr = m_dMu + d * m_nLabelNum;
		eta_ptr = m_dEta + k;
		gnd_eta = m_dEta[y * m_nK + k];
		while(t--)
		{
			dval += (*mu_ptr++) * (gnd_eta - *eta_ptr);
			eta_ptr += m_nK;
		}
		dval /= double(dlen);
		if (dval > OVERFLOW_THREHOLD || dval < -OVERFLOW_THREHOLD) flag = true;
		mrg[k] = dval;
	}
	return flag;
}

void GibbsMedLDA::compute_theta(Stats* ss, int d)
{
	if (!m_dTheta)
		m_dTheta = new double[m_nK];	

	const double DIV = m_dAlpha * m_nK + ss->corpus->docs[d].length;
	int* nd_ptr = ss->nd[d];
	double* theta_ptr = m_dTheta;
	int k = m_nK;
	while(k--)
		*theta_ptr++ = double (*nd_ptr++ + m_dAlpha) / DIV; 
}

bool GibbsMedLDA::compute_phi(Stats* st)
{
	if ( st->nV > m_nV)
	{
		printf(" ERROR> V illeagle\n");	
		return false;
	}

	if(!m_dPhi)
	{
		m_dPhi = new double*[m_nK];
		for(int k = 0; k < m_nK; ++k)
			m_dPhi[k] = new double[m_nV];
	}

	const double Vbeta	= m_nV * m_dBeta;
	int t, nwsum;
	double* phi_ptr = NULL;
	int* nw_ptr = NULL;
	for(int k = 0; k < m_nK; ++k)
	{
		phi_ptr 	= m_dPhi[k];
		nw_ptr	= st->nw[k];
		const double div	= st->nwsum[k] + Vbeta;
		for (t = 0; t < st->nV; ++t)
			*phi_ptr++ = double (*nw_ptr++ + m_dBeta) / div; 
		for (; t < m_nV; ++t)
			*phi_ptr++ = m_dBeta / div;
	}
	return true;
}


double GibbsMedLDA::compute_lhood(Stats* st)
{
	double lhood = 0.0;
	double dlhood = 0, dlword, theta_k;
	int term, k, n;
	int *nd_ptr;
	bool log_space_flag = false;

	compute_phi(st);
	for(int d = 0; d < st->nD; ++d)
	{
		dlhood = 0;
		Document& dc = st->corpus->docs[d];	
		log_space_flag = compute_mrgterm(d, dc.gndlabel, dc.length, m_mrg);
		compute_theta(st, d);
		for(k = 0; k < m_nK; ++k)
			dlhood += m_dTheta[k] * m_mrg[k] * dc.length;

		if (log_space_flag)
		{
			for(k = 0; k < m_nK; ++k)
				m_dTheta[k] = log(m_dTheta[k]) + m_mrg[k];
			for (n = 0; n < dc.length; ++n)
			{
				term = dc.words[n];	
				dlword = 0;
				for(k = 0; k < m_nK; ++k)
					dlword = log_sum(dlword, log(m_dPhi[k][term]) + m_dTheta[k]);
				dlhood -= dlword;
			}
		}
		else
		{
			for(k = 0; k < m_nK; ++k)
				m_dTheta[k] *= exp(m_mrg[k]);
			for (n = 0; n < dc.length; ++n)
			{
				dlword = 0;
				term = dc.words[n];	
				for(k = 0; k < m_nK; ++k)
					dlword += m_dPhi[k][term] * m_dTheta[k];
				dlhood -= log(dlword);
			}
		}
		lhood += dlhood;
	}
	return lhood;
}


double GibbsMedLDA::compute_lhood(Document* dc, double** phi, double* theta)
{
	double dlhood = 0, dlword;
	int term;
	for (int n = 0; n < dc->length; ++n)
	{
		dlword = 0;
		term = dc->words[n];	
		for(int k = 0; k < m_nK; ++k)
			dlword += phi[k][term] * theta[k];
		dlhood -= log(dlword);
	}
	return dlhood;
}

int GibbsMedLDA::gibbs_sampling(Stats* trn_ss, Stats* tst_ss, const int d, const int n)
{
	double Vbeta		= m_dBeta * m_nV;

	Document& dc		= tst_ss->corpus->docs[d];
	const int term		= dc.words[n];
	int topic 			= tst_ss->z[d][n];
	int **m_infnw		= tst_ss->nw;
	int *m_infnwsum	= tst_ss->nwsum;
	int **m_infnd		= tst_ss->nd;

	int **m_nw			= trn_ss->nw;
	int *m_nwsum		= trn_ss->nwsum;
	int **m_nd			= trn_ss->nd;


	m_infnw[topic][term] -= 1;
	m_infnwsum[topic]		-= 1;
	m_infnd[d][topic]		-= 1;

	if ( term < trn_ss->nV)
	{
		m_dPr[0] = (m_nw[0][term] + m_infnw[0][term] + m_dBeta) / (m_nwsum[0] + m_infnwsum[0] + Vbeta) * (m_infnd[d][0] + m_dAlpha);
		for(int k = 1; k < m_nK; ++k)
			m_dPr[k] = m_dPr[k-1] + (m_nw[k][term] + m_infnw[k][term] + m_dBeta) / (m_nwsum[k] + m_infnwsum[k] + Vbeta) * (m_infnd[d][k] + m_dAlpha);
	}
	else 
	{
		m_dPr[0] = (m_infnw[0][term] + m_dBeta) / (m_infnwsum[0] + Vbeta) * (m_infnd[d][0] + m_dAlpha); 
		for(int k = 1; k < m_nK; ++k)
			m_dPr[k] = m_dPr[k-1] + (m_infnw[k][term] + m_dBeta) / (m_infnwsum[k] + Vbeta) * (m_infnd[d][k] + m_dAlpha);
	}

	double rd = myrand() * m_dPr[m_nK-1];
	for(topic = 0; m_dPr[topic] < rd; ++topic);

	m_infnw[topic][term]	+= 1;
	m_infnwsum[topic]		+= 1;
	m_infnd[d][topic]		+= 1;

	return topic;
}

int GibbsMedLDA::predict(double *theta)
{
	int pred_label = -1;		
	double dMax = 0.0;
	double dScore = 0.0;
	int k;

	double *ptr = NULL;
	double *eta_ptr = m_dEta;

	for(int y = 0; y < m_nLabelNum; ++y)
	{
		dScore = 0;										
		k = m_nK;
		ptr = theta;
		while(k--)
			dScore += (*eta_ptr++) * (*ptr++); 
		dScore -= m_dB;
		if ( -1 == pred_label || dScore > dMax) {
			pred_label = y;
			dMax = dScore;
		}
	}
	return pred_label;
}
int GibbsMedLDA::predict(int* topics, int doc_len, int num_iters)
{
	const double Kalpha = m_nK * m_dAlpha;

	int pred_label = -1;		
	double dMax = 0.0;
	double dScore = 0.0;
	int k;

	int *nd_ptr = NULL;
	double* eta_ptr = m_dEta;

	for(int y = 0; y < m_nLabelNum; ++y)
	{
		dScore = 0;										
		k = m_nK;
		nd_ptr = topics;
		while(k--)
			dScore += (*eta_ptr++) * (double (*nd_ptr++) / double(num_iters) + m_dAlpha) / (doc_len + Kalpha); 
		dScore -= m_dB;
		if ( -1 == pred_label || dScore > dMax)
		{
			pred_label = y;
			dMax = dScore;
		}
	}
	return pred_label;
}

int GibbsMedLDA::predict(Stats* ss, const int d)
{
	const int len	= ss->corpus->docs[d].length;
	const double Kalpha = m_nK * m_dAlpha;

	int pred_label = -1;		
	double dMax = 0.0;
	double dScore = 0.0;
	int k;

	int *ND_PTR = ss->nd[d];
	int *nd_ptr = NULL;
	double* eta_ptr = m_dEta;

	for(int y = 0; y < m_nLabelNum; ++y)
	{
		dScore = 0;										
		k = m_nK;
		nd_ptr = ND_PTR;
		while(k--)
			dScore += (*eta_ptr++) * (*nd_ptr++ + m_dAlpha) / (len + Kalpha);
		dScore -= m_dB;
		if ( -1 == pred_label || dScore > dMax)
		{
			pred_label = y;
			dMax = dScore;
		}
	}
	return pred_label;
}

void GibbsMedLDA::m_step()
{
	svmStructSolver();
}

void GibbsMedLDA::svmStructSolver()
{
	LEARN_PARM learn_parm;
	KERNEL_PARM kernel_parm;
	STRUCT_LEARN_PARM struct_parm;
	STRUCTMODEL structmodel;
	int alg_type;

	/* set the parameters. */
	set_init_svm_param(&struct_parm, &learn_parm, &kernel_parm, &alg_type);
	struct_parm.C = m_dC;

	// output the features
	char buff[512];
	sprintf(buff, "%s/Feature.txt", m_params->dir);
	outputLowDimData(m_ptrnstats, buff);

	/* read the training examples */
	SAMPLE sample = read_struct_examples(buff, &struct_parm);

	if(m_params->SVM_ALGTYPE == 0)
		svm_learn_struct(sample, &struct_parm, &learn_parm, &kernel_parm, &structmodel, NSLACK_ALG);
	else if(m_params->SVM_ALGTYPE == 2) 
	{
		struct_parm.C = m_dC * m_ptrnstats->nD;  // Note: in n-slack formulation, C is not divided by N.
		svm_learn_struct_joint(sample, &struct_parm, &learn_parm, &kernel_parm, &structmodel, ONESLACK_PRIMAL_ALG);
	}
	else exit(1);

	/* get the optimal lagrangian multipliers. 
	*    Note: for 1-slack formulation: the "marginalization" is 
	*           needed for fast computation.
	*/
	int nVar = m_ptrnstats->nD * m_nLabelNum;
	memset(m_dMu, 0, sizeof(double) * nVar);

	if ( m_params->SVM_ALGTYPE == 0 ) 
	{
		for (int k = 1; k < structmodel.svm_model->sv_num; k++ ) 
		{
			int n = structmodel.svm_model->supvec[k]->docnum;
			int docnum = structmodel.svm_model->supvec[k]->orgDocNum;
			m_dMu[docnum] = structmodel.svm_model->alpha[k];
		}
	} 
	else if ( m_params->SVM_ALGTYPE == 2 ) 
	{
		for (int k = 1; k < structmodel.svm_model->sv_num; k++ ) 
		{
			int *vecLabel = structmodel.svm_model->supvec[k]->lvec;
			double dval = structmodel.svm_model->alpha[k] / (double) m_ptrnstats->nD; 
			for ( int d = 0; d < m_ptrnstats->nD; d++ ) 
			{
				int label = vecLabel[d];
				m_dMu[d * m_nLabelNum + label] += dval;
			}
		}
	} else ;


#ifdef DEBUG
	FILE *fileptr_mu = fopen("MuSolution.txt", "a");
	for ( int i=0; i < m_ptrnstats->nD; i++ ) 
	{
		for ( int k=0; k < m_nLabelNum; k++ ) 
		{
			int muIx = i * m_nLabelNum + k;
			if ( m_dMu[muIx] > 0 ) 
				fprintf(fileptr_mu, "%d:%.5f ", k, m_dMu[muIx]);
		}
		fprintf(fileptr_mu, "\n");
	}
	fclose(fileptr_mu);

	FILE *fileptr_eta = fopen("SVMLightSolution.txt", "a");
	m_dB = structmodel.svm_model->b;
	for ( int y = 0; y < m_nLabelNum; y++ ) 
	{
		for ( int k=0; k<m_nK; k++ )
		{
			int etaIx = y * m_nK + k;
			m_dEta[etaIx] = structmodel.w[etaIx+1];
			fprintf(fileptr_eta, "%5.5f:%5.5f ", m_dEta[etaIx], structmodel.w[etaIx+1]);
		}
		fprintf(fileptr_eta, "\n");
	}
	fprintf(fileptr_eta, "\n%5.5f\n\n", m_dB);
	fclose(fileptr_eta);
#else
	m_dB = structmodel.svm_model->b;
	for ( int y = 0; y < m_nLabelNum; y++ ) 
	{
		for ( int k = 0; k<m_nK; k++ )
		{
			int etaIx = y * m_nK + k;
			m_dEta[etaIx] = structmodel.w[etaIx+1];
		}
	}
#endif
	m_dsvm_primalobj = structmodel.primalobj;

	// free the memory
	free_struct_sample(sample);
	free_struct_model(structmodel);
}

void GibbsMedLDA::outputLowDimData(Stats* ss, char *filename)
{
	double val;
	double Kalpha = m_nK * m_dAlpha; 
	FILE *fileptr = fopen(filename, "w");
	for ( int d = 0; d < m_ptrnstats->nD; d++ ) 
	{
		Document& dc = ss->corpus->docs[d];
		fprintf(fileptr, "%d %d", m_nK, dc.gndlabel);

		//int *ep = ss->exp[d];
		int* zz = ss->nd[d];
		for ( int k = 0; k < m_nK; k++ ) 
		{
			//val = double(ep[k]) / double(dc.length * m_params->NITERS_APPX);
			val = (double(zz[k]) + m_dAlpha) / (double(dc.length) + Kalpha);
			fprintf(fileptr, " %d:%.10f", k, val);
		}
		fprintf(fileptr, "\n");
	}
	fclose(fileptr);
}

void GibbsMedLDA::set_init_svm_param(STRUCT_LEARN_PARM *struct_parm, LEARN_PARM *learn_parm, KERNEL_PARM *kernel_parm, int *alg_type)
{
	/* set default */
	(*alg_type) = DEFAULT_ALG_TYPE;
	struct_parm->C = -0.01;
	struct_parm->slack_norm = 1;
	struct_parm->epsilon = DEFAULT_EPS;
	struct_parm->custom_argc = 0;
	struct_parm->loss_function = DEFAULT_LOSS_FCT;
	struct_parm->loss_type = DEFAULT_RESCALING;
	struct_parm->newconstretrain = 100;
	struct_parm->ccache_size = 5;
	struct_parm->batch_size = 100;

	struct_parm->delta_ell = COST_PARAMETER; // Simply set to 1 by Qixia

	strcpy(learn_parm->predfile, "trans_predictions");
	strcpy(learn_parm->alphafile, "");
	verbosity = 0;/*verbosity for svm_light*/
	struct_verbosity = 1; /*verbosity for struct learning portion*/
	learn_parm->biased_hyperplane = 1;
	learn_parm->remove_inconsistent = 0;
	learn_parm->skip_final_opt_check = 0;
	learn_parm->svm_maxqpsize = 10;
	learn_parm->svm_newvarsinqp = 0;
	learn_parm->svm_iter_to_shrink = -9999;
	learn_parm->maxiter = 100000;
	learn_parm->kernel_cache_size = 40;
	learn_parm->svm_c = 99999999;  /* overridden by struct_parm->C */
	learn_parm->eps = 0.001;       /* overridden by struct_parm->epsilon */
	learn_parm->transduction_posratio = -1.0;
	learn_parm->svm_costratio = 1.0;
	learn_parm->svm_costratio_unlab = 1.0;
	learn_parm->svm_unlabbound = 1E-5;
	learn_parm->epsilon_crit = 0.001;
	learn_parm->epsilon_a = 1E-10;  /* changed from 1e-15 */
	learn_parm->compute_loo = 0;
	learn_parm->rho = 1.0;
	learn_parm->xa_depth = 0;
	kernel_parm->kernel_type = 0;
	kernel_parm->poly_degree = 3;
	kernel_parm->rbf_gamma = 1.0;
	kernel_parm->coef_lin = 1;
	kernel_parm->coef_const = 1;
	strcpy(kernel_parm->custom,"empty");

	if(learn_parm->svm_iter_to_shrink == -9999) {
		learn_parm->svm_iter_to_shrink=100;
	}

	if((learn_parm->skip_final_opt_check) 
		&& (kernel_parm->kernel_type == LINEAR)) {
			printf("\nIt does not make sense to skip the final optimality check for linear kernels.\n\n");
			learn_parm->skip_final_opt_check=0;
	}    
	if((learn_parm->skip_final_opt_check) 
		&& (learn_parm->remove_inconsistent)) {
			printf("\nIt is necessary to do the final optimality check when removing inconsistent \nexamples.\n");
			exit(0);
	}    
	if((learn_parm->svm_maxqpsize<2)) {
		printf("\nMaximum size of QP-subproblems not in valid range: %ld [2..]\n",learn_parm->svm_maxqpsize); 
		exit(0);
	}
	if((learn_parm->svm_maxqpsize<learn_parm->svm_newvarsinqp)) {
		printf("\nMaximum size of QP-subproblems [%ld] must be larger than the number of\n",learn_parm->svm_maxqpsize); 
		printf("new variables [%ld] entering the working set in each iteration.\n",learn_parm->svm_newvarsinqp); 
		exit(0);
	}
	if(learn_parm->svm_iter_to_shrink<1) {
		printf("\nMaximum number of iterations for shrinking not in valid range: %ld [1,..]\n",learn_parm->svm_iter_to_shrink);
		exit(0);
	}
	if(((*alg_type) < 0) || (((*alg_type) > 5) && ((*alg_type) != 9))) {
		printf("\nAlgorithm type must be either '0', '1', '2', '3', '4', or '9'!\n\n");
		exit(0);
	}
	if(learn_parm->transduction_posratio>1) {
		printf("\nThe fraction of unlabeled examples to classify as positives must\n");
		printf("be less than 1.0 !!!\n\n");
		exit(0);
	}
	if(learn_parm->svm_costratio<=0) {
		printf("\nThe COSTRATIO parameter must be greater than zero!\n\n");
		exit(0);
	}
	if(struct_parm->epsilon<=0) {
		printf("\nThe epsilon parameter must be greater than zero!\n\n");
		exit(0);
	}
	if((struct_parm->ccache_size<=0) && ((*alg_type) == 4)) {
		printf("\nThe cache size must be at least 1!\n\n");
		exit(0);
	}
	if(((struct_parm->batch_size<=0) || (struct_parm->batch_size>100))  
		&& ((*alg_type) == 4)) {
			printf("\nThe batch size must be in the interval ]0,100]!\n\n");
			exit(0);
	}
	if((struct_parm->slack_norm<1) || (struct_parm->slack_norm>2)) {
		printf("\nThe norm of the slacks must be either 1 (L1-norm) or 2 (L2-norm)!\n\n");
		exit(0);
	}
	if((struct_parm->loss_type != SLACK_RESCALING) 
		&& (struct_parm->loss_type != MARGIN_RESCALING)) {
			printf("\nThe loss type must be either 1 (slack rescaling) or 2 (margin rescaling)!\n\n");
			exit(0);
	}
	if(learn_parm->rho<0) {
		printf("\nThe parameter rho for xi/alpha-estimates and leave-one-out pruning must\n");
		printf("be greater than zero (typically 1.0 or 2.0, see T. Joachims, Estimating the\n");
		printf("Generalization Performance of an SVM Efficiently, ICML, 2000.)!\n\n");
		exit(0);
	}
	if((learn_parm->xa_depth<0) || (learn_parm->xa_depth>100)) {
		printf("\nThe parameter depth for ext. xi/alpha-estimates must be in [0..100] (zero\n");
		printf("for switching to the conventional xa/estimates described in T. Joachims,\n");
		printf("Estimating the Generalization Performance of an SVM Efficiently, ICML, 2000.)\n");
		exit(0);
	}
	parse_struct_parameters(struct_parm);
}

void GibbsMedLDA::save_results(double tAvgEnt)
{
	printf(">>> Saving results & evaluating performace\n");
	char buf[512];

	int nAcc = 0;
	int total = 0;
	double lhood = 0;
	for(int d = 0; d < m_ptststats->nD; ++d)
	{
		Document &dc = m_ptststats->corpus->docs[d];
		lhood	+= dc.lhood;
		total += dc.length;
		if (dc.gndlabel == dc.predlabel)
			++nAcc;
	}
	double dAcc = (double) nAcc / m_ptststats->nD;
	double perwordlhood = lhood / double(total);
	printf("Accuracy: %5.5f\n", dAcc);

	sprintf(buf, "%s/evl-performance.txt", m_params->dir);
	FILE* fileptr = fopen(buf, "w");
	fprintf(fileptr, "num docs: %d\n", m_ptststats->nD);
	fprintf(fileptr, "num terms: %d\n", m_nV);
	fprintf(fileptr, "num topics: %d\n", m_nK);
	fprintf(fileptr, "cost parameter l: %d\n", (int)COST_PARAMETER);
	fprintf(fileptr, "Alpha: %5.10f\n", (m_dAlpha*m_nK)); 
	//fprintf(fileptr, "Sample Lag: %d\n", m_params->SAMPLING_LAG);
	fprintf(fileptr, "C:	%5.10f\n\n", m_params->INITIAL_C);
	fprintf(fileptr, "Accuracy: %5.10f\n", dAcc);
	fprintf(fileptr, "Likelihood: %5.10f\n", lhood);
	fprintf(fileptr, "PerWordLikelihood: %5.10f\n", perwordlhood);
	fprintf(fileptr, "testAverageEntropy: %5.10f\n", tAvgEnt);
	fclose(fileptr);

	sprintf(buf, "%s/pred_wordassigned.txt", m_params->dir);
	m_ptststats->save_wordassigned(buf);
	sprintf(buf, "%s/pred_results.txt", m_params->dir);
	m_ptststats->save_label_only(buf);

	sprintf(buf, "%s/gml_%dtopic_test.txt", m_params->dir, m_params->NTOPICS);
	FILE* fout = fopen(buf, "w");
	if (!fout) return;
	fprintf(fout, "%d\n", m_ptststats->nD);
	for(int d = 0; d < m_ptststats->nD; ++d)
	{
		Document &dc = m_ptststats->corpus->docs[d];
		fprintf(fout, "%d %d", m_nK, dc.gndlabel);
		compute_theta(m_ptststats, d);	
		for (int k = 0; k < m_nK; ++k)
			fprintf(fout, " %d:%.10f", k, m_dTheta[k]);
		fprintf(fout, "\n");
	}
	fclose(fout);
	printf(">>> Saving results & evaluating performace finished! \n");
}

void GibbsMedLDA::save_model_property(Stats* st, Params *param)
{
	char buf[512];	
	sprintf(buf, "%s/model_property.txt", param->dir);
	FILE *file = fopen(buf, "w");
	int d, k, t;
	int nZero = 0;
	compute_phi(st);
	for(k = 0; k < m_nK; ++k)
		for(t = 0; t < m_nV; ++t)
			if (m_dPhi[k][t] < NUMERICAL_ZERO) nZero++;
	double dZeroPercent = double(nZero) / (m_nK * m_nV);
	fprintf(file, "ZeroPercentage: %.10f\n", dZeroPercent);

	double *theta = NULL;
	double dEntropy = 0;
	double entropy = 0;
	double val;

	sprintf(buf, "%s/theta_train.txt", param->dir);
	FILE *thetaFile = fopen(buf, "w");

	for(d = 0; d < st->nD; ++d){
		compute_theta(st, d);
		theta = m_dTheta;
		k = m_nK;
		dEntropy = 0;
		while(k--){
			val = *theta++;
			fprintf(thetaFile, "%.10f ", val);
			if (val > 0)
				dEntropy -= val * log(val);
		}
		fprintf(thetaFile, "\n");
		entropy += dEntropy;
	}
	fclose(thetaFile);
	entropy /= double(st->nD);
	fprintf(file, "AverageEntropy: %.10f\n", entropy);

	int numNonZero = 0;
	int NN = m_ptrnstats->nD * m_nLabelNum;
	for (int j = 0; j < NN; ++j) {
		if (m_dMu[j] != 0)
			numNonZero ++;
	}
	fprintf(file, "Lagrange multipliers non-zero percentage: %lf\n", double(numNonZero)/double(NN));

	fclose(file);
}

void GibbsMedLDA::save_model(const char* model_file)
{
	printf(">>> Saving model ...\n");
	FILE* file = fopen(model_file, "w");
	if (!file) {
		printf("ERROR> Cannot open model file error !");	
		return;
	}

	fprintf(file, "%f\n", m_params->INITIAL_C);

	fprintf(file, "%.10f\n", m_dB);
	double* ptr = m_dEta;
	int N = m_nLabelNum * m_nK;
	while(N--)
		fprintf(file, "%5.10f ", *ptr++); 
	fprintf(file, "\n");

	//m_ptrnstats->save_wordassigned(file);
	for(int i = 0; i < m_nK; ++i){
		for(int j = 0; j < m_nV; ++j){
			fprintf(file, "%lf ", m_dPhi[i][j]);
		}
		fprintf(file, "\n");
	}

	ptr = m_dMu;
	N = m_ptrnstats->nD * m_nLabelNum;
	while(N--)
		fprintf(file, "%5.10f ", *ptr++);
	fprintf(file, "\n");
	fclose(file);
	printf(">>> Savning model finished !\n");
}

void GibbsMedLDA::load_model(const char* model_file)
{
	printf(">>> Loading model ...\n");
	FILE* file = fopen(model_file, "r");
	if (!file)
	{
		printf("ERROR> Cannot open model file error !");	
		return;
	}

	// Loading C
	fscanf(file, "%f\n", &(m_params->INITIAL_C));
	fprintf(file, "%lf\n", &m_dB);

	// Loading eta (svm classifier)
	int N = m_params->NLABELS * m_params->NTOPICS;
	m_dEta = new double[N];
	for(int i = 0; i < N; ++i)
		fscanf(file, "%lf", &m_dEta[i]);

	// Loading GibbsMedLDA model
	if (m_ptrnstats) delete m_ptrnstats;
	m_ptrnstats = new Stats();
	m_ptrnstats->load(file, m_params);
	m_nV = m_ptrnstats->nV;

	N = m_ptrnstats->nD * m_params->NLABELS;
	m_dMu	= new double[N];
	for(int i = 0; i < N; ++i)
		fscanf(file, "%lf", &m_dMu[i]);

	fclose(file);
	printf(">>> Loading model finished! \n");
}
