#ifndef _GIBBS_MED_LDA_H
#define _GIBBS_MED_LDA_H

#include "defs.h"
#include "corpus.h"
#include "params.h"
#include <stdlib.h>
#include <stdio.h>
#include <iostream>
#include <string.h>
#include <time.h>

#include "svm_multiclass/svm_struct_api.h"
#include "svm_multiclass/svm_struct_learn.h"
#include "svm_multiclass/svm_struct_common.h"


using namespace std;

class TopK
{
public:
	TopK(int len): length(len), key(NULL), value(NULL)
	{
		key = new int[length];
		value = new double[length];
	}
	~TopK()
	{
		if (key) delete[] key;
		if (value) delete[] value;
	}
	void reset();
	void push_back(int id, double v);
	void sort();

private:
	void shiftdown(int pos, int rg);
	inline void exchange(int p1, int p2);

public:
	int length;
	int* key;
	double* value;
};

class Stats
{
public:
	Stats();
	~Stats();

	void new_stats(Corpus *cps, Params* param);
	void load(const char *filename, Params* param);
	void load(FILE* file, Params* param);
	void free();

	void save_wordassigned(const char *filename);
	void save_wordassigned(FILE *file);
	void save_topwords(const char* filename, Params* param);
	void save_label_only(const char* filename);

public:
	int nD; // # docs
	int nV; // # terms
	int nK; // # topics

	int **z;	// topic assignments for D X doc.lenght
	int **nw; // m_nw[i][j]: number of instances of word/term i assigned to topic j, size K x V
	int *nwsum; // m_nwsum[k]: # words assigned to the topic k, 1 X K.
	int **nd; // m_nd[i][j]: number of words in document i assigned to topic j, size m_D x K
	
	Corpus* corpus;
};

class GibbsMedLDA
{
public:
	GibbsMedLDA(Params *params): m_params(params) 
	{ 
		if (!m_params)
		{
			printf("ERROR> NO params error!\n");
			exit(1);
		}
		set_default_values();
	}
	~GibbsMedLDA();

/*
*	sharing	
*/

public:
	void set_default_values();

	void save_model(const char* model_file);
	void load_model(const char* model_file);
	void save_model_property(Stats* st, Params *param);

	int predict(Stats* ss, const int d);
	int predict(int* topics, int doc_len, int num_iters);
	int predict(double* theta);

private:
	void compute_theta(Stats* ss, int d);
	bool compute_phi(Stats* st);

/*
*	training 
*/

public:
	void init_est(); 
	void init_est(Corpus* corpus);
	void init_estc(const char* prev_state_file);
	void run_em();
	double e_step(int em_it);
	void m_step();

private:
	int gibbs_sampling(Stats* ss, const double* mrg, const int d, const int n);
	int gibbs_sampling_logspace(Stats* ss, const double* mrg, const int d, const int n);
	inline bool compute_mrgterm(const int d, const int y, const int dlen, double* mrg);
	double compute_lhood(Stats* st);

	void outputLowDimData(Stats* ss, char *filename);
	void svmStructSolver();
	void set_init_svm_param(STRUCT_LEARN_PARM *struct_parm, 
									LEARN_PARM *learn_parm, 
									KERNEL_PARM *kernel_parm, 
									int *alg_type);
/*
*	 prediction
*/

public:
	void init_inf(Corpus* corpus);
	void init_inf(const char* model_file);
	double inference(double *tAvgEnt);
	void save_results(double tAvgEnt);
	//void inference_semi();

private:
	int gibbs_sampling(Stats* trn_ss, Stats* tst_ss, const int d, const int n);
	double compute_lhood(Document* doc, double** phi, double* theta);
	//bool compute_phi(Stats* st, Stats* sp);
	//double compute_lhood(Stats* st, Stats* sp);


public:
	Params* m_params;
	Stats* m_ptrnstats;
	Stats* m_ptststats;

	int m_nK;
	int m_nV;
	int m_nLabelNum; // number of classes, |C|
	double m_dC;
	double m_dAlpha; 
	double m_dBeta;

	double *m_dMu;  // D X #labels
	double *m_dEta; // #labels * #topics
	double *m_dPr; // temp variable for sampling
	double *m_mrg; // 1 X #topics

	double *m_dTheta; // 1 \times #topics
	double **m_dPhi; // only used when calculating likelihood;
						  // #topic * #terms

	double m_dsvm_primalobj;
	double m_dB;    // bias of SVM solver
};

#endif
